@@ -8,7 +8,7 @@ guess = function(x) {
   var
 }
 
-dcast <- function(
+dcast = function(
   data, formula, fun.aggregate = NULL, ..., margins = NULL,
   subset = NULL, fill = NULL, value.var = guess(data)
 ) {
@@ -115,7 +115,7 @@ aggregate_funs = function(funs, vals, sep="_", ...) {
     if (is.null(nm) || !nzchar(nm)) {
       nm = all.names(funs[[i]], max.names=1L, functions=TRUE)
     }
-    if (!length(nm)) nm <- paste0("fun", i)
+    if (!length(nm)) nm = paste0("fun", i)
     construct_funs(funs[i], nm, vals[[i]])
   })
   as.call(c(quote(list), unlist(ans)))
@@ -185,12 +185,8 @@ dcast.data.table = function(data, formula, fun.aggregate = NULL, sep = "_", ...,
     fun.call = aggregate_funs(fun.call, lvals, sep, ...)
     maybe_err = function(list.of.columns) {
       if (!all(lengths(list.of.columns) == 1L)) {
-        msg <- gettext("Aggregating functions should take a vector as input and return a single value (length=1), but they do not, so the result is undefined. Please fix by modifying your function so that a single value is always returned.")
-        if (is.null(fill)) { # TODO change to always stopf #6329
-          stop(msg, domain=NA, call. = FALSE)
-        } else {
-          warning(msg, domain=NA, call. = FALSE)
-        }
+        msg = gettext("Aggregating functions should take a vector as input and return a single value (length=1), but they do not, so the result is undefined. Please fix by modifying your function so that a single value is always returned.")
+        stop(msg, domain=NA, call. = FALSE)
       }
       list.of.columns
     }
@@ -214,42 +210,42 @@ dcast.data.table = function(data, formula, fun.aggregate = NULL, sep = "_", ...,
   # 'dat' != 'data'? then setkey to speed things up (slightly), else ad-hoc (for now). Still very fast!
   if (!is.null(fun.call) || !is.null(subset))
     setkeyv(dat, varnames)
-  if (length(rhsnames)) {
-    lhs = shallow(dat, lhsnames); rhs = shallow(dat, rhsnames); val = shallow(dat, valnames)
-    # handle drop=TRUE/FALSE - Update: Logic moved to R, AND faster than previous version. Take that... old me :-).
-    if (all(drop)) {
-      map = setDT(lapply(list(lhsnames, rhsnames), function(cols) frankv(dat, cols=cols, ties.method="dense", na.last=FALSE))) # #2202 fix
-      maporder = lapply(map, order_)
-      mapunique = lapply(seq_along(map), function(i) .Call(CsubsetVector, map[[i]], maporder[[i]]))
-      lhs = .Call(CsubsetDT, lhs, maporder[[1L]], seq_along(lhs))
-      rhs = .Call(CsubsetDT, rhs, maporder[[2L]], seq_along(rhs))
-    } else {
-      lhs_ = if (!drop[1L]) cj_uniq(lhs) else setkey(unique(lhs, by=names(lhs)))
-      rhs_ = if (!drop[2L]) cj_uniq(rhs) else setkey(unique(rhs, by=names(rhs)))
-      map = vector("list", 2L)
-      .Call(Csetlistelt, map, 1L, lhs_[lhs, which=TRUE])
-      .Call(Csetlistelt, map, 2L, rhs_[rhs, which=TRUE])
-      setDT(map)
-      mapunique = vector("list", 2L)
-      .Call(Csetlistelt, mapunique, 1L, seq_len(nrow(lhs_)))
-      .Call(Csetlistelt, mapunique, 2L, seq_len(nrow(rhs_)))
-      lhs = lhs_; rhs = rhs_
-    }
-    maplen = lengths(mapunique)
-    idx = do.call(CJ, mapunique)[map, 'I' := .I][["I"]] # TO DO: move this to C and avoid materialising the Cross Join.
-    some_fill = anyNA(idx)
-    fill.default = if (run_agg_funs && is.null(fill) && some_fill) dat_for_default_fill[, maybe_err(eval(fun.call))]
-    if (run_agg_funs && is.null(fill) && some_fill) {
-      fill.default = dat_for_default_fill[0L][, maybe_err(eval(fun.call))]
-    }
-    ans = .Call(Cfcast, lhs, val, maplen[[1L]], maplen[[2L]], idx, fill, fill.default, is.null(fun.call), some_fill)
-    allcols = do.call(paste, c(rhs, sep=sep))
-    if (length(valnames) > 1L)
-      allcols = do.call(paste, if (identical(".", allcols)) list(valnames, sep=sep)
-            else c(CJ(valnames, allcols, sorted=FALSE), sep=sep))
-      # removed 'setcolorder()' here, #1153
-    setattr(ans, 'names', c(lhsnames, allcols))
-    setDT(ans); setattr(ans, 'sorted', lhsnames)
-  } else internal_error("empty rhsnames") # nocov
-  return(ans)
+  if (!length(rhsnames)) internal_error("empty rhsnames") # nocov
+  lhs = shallow(dat, lhsnames); rhs = shallow(dat, rhsnames); val = shallow(dat, valnames)
+  # handle drop=TRUE/FALSE - Update: Logic moved to R, AND faster than previous version. Take that... old me :-).
+  if (all(drop)) {
+    map = setDT(lapply(list(lhsnames, rhsnames), function(cols) frankv(dat, cols=cols, ties.method="dense", na.last=FALSE))) # #2202 fix
+    maporder = lapply(map, order_)
+    mapunique = lapply(seq_along(map), function(i) .Call(CsubsetVector, map[[i]], maporder[[i]]))
+    lhs = .Call(CsubsetDT, lhs, maporder[[1L]], seq_along(lhs))
+    rhs = .Call(CsubsetDT, rhs, maporder[[2L]], seq_along(rhs))
+  } else {
+    lhs_ = if (!drop[1L]) cj_uniq(lhs) else setkey(unique(lhs, by=names(lhs)))
+    rhs_ = if (!drop[2L]) cj_uniq(rhs) else setkey(unique(rhs, by=names(rhs)))
+    map = vector("list", 2L)
+    .Call(Csetlistelt, map, 1L, lhs_[lhs, which=TRUE])
+    .Call(Csetlistelt, map, 2L, rhs_[rhs, which=TRUE])
+    setDT(map)
+    mapunique = vector("list", 2L)
+    .Call(Csetlistelt, mapunique, 1L, seq_len(nrow(lhs_)))
+    .Call(Csetlistelt, mapunique, 2L, seq_len(nrow(rhs_)))
+    lhs = lhs_; rhs = rhs_
+  }
+  maplen = lengths(mapunique)
+  idx = do.call(CJ, mapunique)[map, 'I' := .I][["I"]] # TO DO: move this to C and avoid materialising the Cross Join.
+  some_fill = anyNA(idx)
+  fill.default = if (run_agg_funs && is.null(fill) && some_fill) dat_for_default_fill[, maybe_err(eval(fun.call))]
+  if (run_agg_funs && is.null(fill) && some_fill) {
+    fill.default = dat_for_default_fill[0L][, maybe_err(eval(fun.call))]
+  }
+  ans = .Call(Cfcast, lhs, val, maplen[[1L]], maplen[[2L]], idx, fill, fill.default, is.null(fun.call), some_fill)
+  allcols = do.call(paste, c(rhs, sep=sep))
+  if (length(valnames) > 1L)
+    allcols = do.call(paste, if (identical(".", allcols)) list(valnames, sep=sep)
+          else c(CJ(valnames, allcols, sorted=FALSE), sep=sep))
+    # removed 'setcolorder()' here, #1153
+  setattr(ans, 'names', c(lhsnames, allcols))
+  setDT(ans)
+  setattr(ans, 'sorted', lhsnames)
+  ans
 }
