@@ -12,14 +12,14 @@ static bool RestoreAfterFork = true;  // see #2885 in v1.12.0
 static int getIntEnv(const char *name, int def)
 {
   const char *val = getenv(name);
-  if (val==NULL) return def;
+  if (val == NULL) return def;
   size_t nchar = strlen(val);
-  if (nchar==0) return def;
+  if (nchar == 0) return def;
   char *end;
   errno = 0;
   long int ans = strtol(val, &end, 10);  // ignores leading whitespace. If it fully consumed the string, *end=='\0' and isspace('\0')==false
   while (isspace(*end)) end++;  // ignore trailing whitespace
-  if (errno || (size_t)(end-val)!=nchar || ans<1 || ans>INT_MAX) {
+  if (errno || (size_t)(end - val) != nchar || ans < 1 || ans > INT_MAX) {
     warning(_("Ignoring invalid %s==\"%s\". Not an integer >= 1. Please remove any characters that are not a digit [0-9]. See ?data.table::setDTthreads."), name, val);
     return def;
   }
@@ -29,52 +29,70 @@ static int getIntEnv(const char *name, int def)
 static inline int imin(int a, int b) { return a < b ? a : b; }
 static inline int imax(int a, int b) { return a > b ? a : b; }
 
-void initDTthreads(void) {
+static inline int cap_threads(int n)
+{
+  n = imin(n, omp_get_thread_limit());  // honors OMP_THREAD_LIMIT when OpenMP started; e.g. CRAN sets this to 2. Often INT_MAX meaning unlimited/unset
+  n = imin(n, omp_get_max_threads());   // honors OMP_NUM_THREADS when OpenMP started, plus reflects any omp_set_* calls made since
+  // max_threads() -vs- num_procs(): https://software.intel.com/en-us/forums/intel-visual-fortran-compiler-for-windows/topic/302866
+  n = imin(n, getIntEnv("OMP_THREAD_LIMIT", INT_MAX));  // user might expect `Sys.setenv(OMP_THREAD_LIMIT=2);setDTthreads()` to work. Satisfy this
+  n = imin(n, getIntEnv("OMP_NUM_THREADS", INT_MAX));   //   expectation by reading them again now. OpenMP just reads them on startup (quite reasonably)
+  n = imax(n, 1);  // just in case omp_get_* returned <=0 for any reason, or the env variables above are set <=0
+  return n;
+}
+
+void initDTthreads(void)
+{
   // called at package startup from init.c
   // also called by setDTthreads(threads=NULL) (default) to reread environment variables; see setDTthreads below
   // No verbosity here in this setter. Verbosity is in getDTthreads(verbose=TRUE)
   int ans = getIntEnv("R_DATATABLE_NUM_THREADS", INT_MIN);
-  if (ans>=1) {
+  if (ans >= 1) {
     ans = imin(ans, omp_get_num_procs());  // num_procs is a hard limit; user cannot achieve more. ifndef _OPENMP then myomp.h defines this to be 1
   } else {
     // Only when R_DATATABLE_NUM_THREADS is unset (or <=0) do we use PROCS_PERCENT; #4514
     int perc = getIntEnv("R_DATATABLE_NUM_PROCS_PERCENT", 50); // use "NUM_PROCS" to use the same name as the OpenMP function this uses
     // 50% of logical CPUs by default; half of 8 is 4 on laptop with 4 cores. Leaves plenty of room for other processes: #3395 & #3298
-    if (perc<=1 || perc>100) {
+    if (perc <= 1 || perc > 100) {
       warning(_("Ignoring invalid R_DATATABLE_NUM_PROCS_PERCENT==%d. If used it must be an integer between 2 and 100. Default is 50. See ?setDTtheads."), perc);
       // not allowing 1 is to catch attempts to use 1 or 1.0 to represent 100%.
       perc = 50;
     }
-    ans = imax(omp_get_num_procs()*perc/100, 1); // imax for when formula would result in 0.
+    ans = imax(omp_get_num_procs() * perc / 100, 1); // imax for when formula would result in 0.
   }
-  ans = imin(ans, omp_get_thread_limit());  // honors OMP_THREAD_LIMIT when OpenMP started; e.g. CRAN sets this to 2. Often INT_MAX meaning unlimited/unset
-  ans = imin(ans, omp_get_max_threads());   // honors OMP_NUM_THREADS when OpenMP started, plus reflects any omp_set_* calls made since
-  // max_threads() -vs- num_procs(): https://software.intel.com/en-us/forums/intel-visual-fortran-compiler-for-windows/topic/302866
-  ans = imin(ans, getIntEnv("OMP_THREAD_LIMIT", INT_MAX));  // user might expect `Sys.setenv(OMP_THREAD_LIMIT=2);setDTthreads()` to work. Satisfy this
-  ans = imin(ans, getIntEnv("OMP_NUM_THREADS", INT_MAX));   //   expectation by reading them again now. OpenMP just reads them on startup (quite reasonably)
-  ans = imax(ans, 1);  // just in case omp_get_* returned <=0 for any reason, or the env variables above are set <=0
-  DTthreads = ans;
+  DTthreads = cap_threads(ans);
   DTthrottle = imax(1, getIntEnv("R_DATATABLE_THROTTLE", 1024)); // 2nd thread is used only when n>1024, 3rd thread when n>2048, etc
 }
 
-int getDTthreads(const int64_t n, const bool throttle) {
+int getDTthreads(const int64_t n, const bool throttle)
+{
   // this is the main getter used by all parallel regions; they specify num_threads(n, true|false).
   // Keep this light, simple and robust. initDTthreads() ensures 1 <= DTthreads <= omp_get_num_proc()
   // throttle introduced in 1.12.10 (see NEWS item); #4484
   // throttle==true  : a number of iterations per thread (DTthrottle) is applied before a second thread is utilized
   // throttle==false : parallel region is already pre-chunked such as in fread; e.g. two batches intended for two threads
-  if (n<1) return 1; // 0 or negative could be deliberate in calling code for edge cases where loop is not intended to run at all
-  int64_t ans = throttle ? 1+(n-1)/DTthrottle :  // 1 thread for n<=1024, 2 thread for n<=2048, etc
+  if (n < 1) return 1; // 0 or negative could be deliberate in calling code for edge cases where loop is not intended to run at all
+  int64_t ans = throttle ? 1 + (n - 1) / DTthrottle :  // 1 thread for n<=1024, 2 thread for n<=2048, etc
                            n;                    // don't use 20 threads for just one or two batches
-  return ans>=DTthreads ? DTthreads : (int)ans;  // apply limit in static local DTthreads saved there by initDTthreads() and setDTthreads()
+  return ans >= DTthreads ? DTthreads : (int)ans;  // apply limit in static local DTthreads saved there by initDTthreads() and setDTthreads()
 }
 
-static const char *mygetenv(const char *name, const char *unset) {
+static const char *mygetenv(const char *name, const char *unset)
+{
   const char *ans = getenv(name);
-  return (ans==NULL || ans[0]=='\0') ? unset : ans;
+  return (ans == NULL || ans[0] == '\0') ? unset : ans;
 }
 
-SEXP getDTthreads_R(SEXP verbose) {
+SEXP getDTthreads_C(SEXP n, SEXP throttle)
+{
+  if(!isInteger(n) || INTEGER(n)[0] < 0)
+    internal_error(__func__, "n must be non-negative integer");  // # nocov
+  if(!IS_TRUE_OR_FALSE(throttle))
+    internal_error(__func__, "throttle must be TRUE or FALSE");  // # nocov
+  return ScalarInteger(getDTthreads(INTEGER(n)[0], LOGICAL(throttle)[0]));
+}
+
+SEXP getDTthreads_R(SEXP verbose)
+{
   if(!IS_TRUE_OR_FALSE(verbose))
     error(_("%s must be TRUE or FALSE"), "verbose");
   if (LOGICAL(verbose)[0]) {
@@ -102,15 +120,16 @@ SEXP getDTthreads_R(SEXP verbose) {
   return ScalarInteger(getDTthreads(INT_MAX, false));
 }
 
-SEXP setDTthreads(SEXP threads, SEXP restore_after_fork, SEXP percent, SEXP throttle) {
+SEXP setDTthreads(SEXP threads, SEXP restore_after_fork, SEXP percent, SEXP throttle)
+{
   if (!isNull(restore_after_fork)) {
-    if (!isLogical(restore_after_fork) || LOGICAL(restore_after_fork)[0]==NA_LOGICAL) {
+    if (!isLogical(restore_after_fork) || LOGICAL(restore_after_fork)[0] == NA_LOGICAL) {
       error(_("restore_after_fork= must be TRUE, FALSE, or NULL (default). getDTthreads(verbose=TRUE) reports the current setting.\n"));
     }
     RestoreAfterFork = LOGICAL(restore_after_fork)[0];  // # nocov
   }
   if (length(throttle)) {
-    if (!isInteger(throttle) || LENGTH(throttle)!=1 || INTEGER(throttle)[0]<1)
+    if (!isInteger(throttle) || LENGTH(throttle) != 1 || INTEGER(throttle)[0] < 1)
       error(_("'throttle' must be a single number, non-NA, and >=1"));
     DTthrottle = INTEGER(throttle)[0];
   }
@@ -123,23 +142,21 @@ SEXP setDTthreads(SEXP threads, SEXP restore_after_fork, SEXP percent, SEXP thro
     // If a CPU has been unplugged (high end servers allow live hardware replacement) then omp_get_num_procs() will
     // reflect that and a call to setDTthreads(threads=NULL) will update DTthreads.
   } else if (length(threads)) {
-    int n=0;
-    if (length(threads)!=1 || !isInteger(threads) || (n=INTEGER(threads)[0]) < 0) {  // <0 catches NA too since NA is negative (INT_MIN)
+    int n = 0;
+    if (length(threads) != 1 || !isInteger(threads) || (n = INTEGER(threads)[0]) < 0) {  // <0 catches NA too since NA is negative (INT_MIN)
       error(_("threads= must be either NULL or a single number >= 0. See ?setDTthreads."));
     }
     int num_procs = imax(omp_get_num_procs(), 1); // max just in case omp_get_num_procs() returns <= 0 (perhaps error, or unsupported)
-    if (!isLogical(percent) || length(percent)!=1 || LOGICAL(percent)[0]==NA_LOGICAL) {
+    if (!isLogical(percent) || length(percent) != 1 || LOGICAL(percent)[0] == NA_LOGICAL) {
       internal_error(__func__, "percent= must be TRUE or FALSE at C level");  // # nocov
     }
     if (LOGICAL(percent)[0]) {
-      if (n<2 || n>100) internal_error(__func__, "threads==%d should be between 2 and 100 (percent=TRUE at C level)", n);  // # nocov
-      n = num_procs*n/100;  // if 0 it will be reset to 1 in the imax() below
+      if (n < 2 || n > 100) internal_error(__func__, "threads==%d should be between 2 and 100 (percent=TRUE at C level)", n);  // # nocov
+      n = num_procs * n / 100;  // if 0 it will be reset to 1 in the imax() below
     } else {
-      if (n==0 || n>num_procs) n = num_procs; // setDTthreads(0) == setDTthread(percent=100); i.e. use all logical CPUs (the default in 1.12.0 and before, from 1.12.2 it's 50%)
+      if (n == 0 || n > num_procs) n = num_procs; // setDTthreads(0) == setDTthread(percent=100); i.e. use all logical CPUs (the default in 1.12.0 and before, from 1.12.2 it's 50%)
     }
-    n = imin(n, omp_get_thread_limit());  // can't think why this might be different from its value on startup, but call it just in case
-    n = imin(n, getIntEnv("OMP_THREAD_LIMIT", INT_MAX));  // user might have called Sys.setenv(OMP_THREAD_LIMIT=) since startup and expect setDTthreads to respect it
-    DTthreads = imax(n, 1);  // imax just in case
+    DTthreads = cap_threads(n);
     // Do not call omp_set_num_threads() here. Any calls to omp_set_num_threads() affect other
     // packages and R itself too which has some OpenMP usage. Instead we set our own DTthreads
     // static variable and read that from getDTthreads(n, throttle).
@@ -171,19 +188,21 @@ SEXP setDTthreads(SEXP threads, SEXP restore_after_fork, SEXP percent, SEXP thro
 
 static int pre_fork_DTthreads = 0;
 
-void when_fork(void) {
+void when_fork(void)
+{
   pre_fork_DTthreads = DTthreads;
   DTthreads = 1;
 }
 
-void after_fork(void) {
+void after_fork(void)
+{
   if (RestoreAfterFork) DTthreads = pre_fork_DTthreads;
 }
 
-void avoid_openmp_hang_within_fork(void) {
+void avoid_openmp_hang_within_fork(void)
+{
   // Called once on loading data.table from init.c
 #ifdef _OPENMP
   pthread_atfork(&when_fork, &after_fork, NULL);
 #endif
 }
-
